
import petsc4py
petsc4py.init()
import petsc4py.PETSc as psc
import numpy as np
from ngsolve import *
from scipy.sparse import csr_matrix
from netgen.geom2d import unit_square

mesh = Mesh(unit_square.GenerateMesh(maxh=0.2))
fes = H1(mesh, order=3, dirichlet="bottom|top|left|right")
u,v = fes.TnT()

f = LinearForm(fes)
f += SymbolicLFI(32 * (y*(1-y)+x*(1-x)) * v)

a = BilinearForm(fes, symmetric=True)
a += SymbolicBFI(grad(u)*grad(v))

a.Assemble()
f.Assemble()

gfu = GridFunction(fes)

fpsc = psc.Vec()
fpsc.createWithArray(f.vec.FV().NumPy())

vals, ind, indptr= a.mat.CSR()
spy_csr = csr_matrix((vals, ind, indptr), shape=(a.mat.height, a.mat.width))

apsc = psc.Mat().createAIJ(size=spy_csr.shape)
apsc.setUp()
(Istart, Iend) = apsc.getOwnershipRange()
ai = spy_csr.indptr[Istart:Iend+1] - spy_csr.indptr[Istart]
aj = spy_csr.indices[spy_csr.indptr[Istart]:spy_csr.indptr[Iend]]
av = spy_csr.data[spy_csr.indptr[Istart]:spy_csr.indptr[Iend]]

apsc.setValuesCSR(ai,aj,av)
apsc.assemble()

ksp = psc.KSP()
ksp.create()
ksp.setOperators(apsc)

upsc = psc.Vec()
upsc.createWithArray(gfu.vec.FV().NumPy())

ksp.solve(fpsc,upsc)

Draw (gfu)
